Package: filearray
Type: Package
Title: File-Backed Array for Out-of-Memory Computation
Version: 0.2.2
Language: en-US
Encoding: UTF-8
License: LGPL-3
URL: https://dipterix.org/filearray/,
        https://github.com/dipterix/filearray
BugReports: https://github.com/dipterix/filearray/issues
Authors@R: c(
    person(
        "Zhengjia", "Wang", role = c("aut", "cre", "cph"),
        email = "dipterix.wang@gmail.com"
    ))
Description: Stores large arrays in files to avoid occupying large
    memories. Implemented with super fast gigabyte-level multi-threaded 
    reading/writing via 'OpenMP'. Supports multiple non-character data 
    types (double, float, complex, integer, logical, and raw).
Imports: digest, fastmap (>= 1.1.1), methods, Rcpp, uuid (>= 1.1.0)
Suggests: bit64, knitr, rmarkdown, testthat (>= 3.0.0)
LinkingTo: BH, Rcpp
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-22 19:18:10 UTC; dipterix
Author: Zhengjia Wang [aut, cre, cph]
Maintainer: Zhengjia Wang <dipterix.wang@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 05:10:37 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-23 09:23:21 UTC; unix
Archs: filearray.so.dSYM
