# Generated by roxygen2: do not edit by hand

S3method("[",FileArray)
S3method("[<-",FileArray)
S3method("[[",FileArray)
S3method("dimnames<-",FileArray)
S3method(as.array,FileArray)
S3method(as_filearray,FileArray)
S3method(as_filearray,FileArrayProxy)
S3method(as_filearray,RAVEFileArray)
S3method(as_filearray,character)
S3method(as_filearray,default)
S3method(as_filearrayproxy,FileArray)
S3method(as_filearrayproxy,FileArrayProxy)
S3method(as_filearrayproxy,default)
S3method(dim,FileArray)
S3method(dimnames,FileArray)
S3method(fwhich,FileArray)
S3method(fwhich,default)
S3method(length,FileArray)
S3method(max,FileArray)
S3method(min,FileArray)
S3method(range,FileArray)
S3method(subset,FileArray)
S3method(sum,FileArray)
export(apply)
export(as_filearray)
export(as_filearrayproxy)
export(filearray_bind)
export(filearray_checkload)
export(filearray_create)
export(filearray_load)
export(filearray_load_or_create)
export(filearray_threads)
export(fmap)
export(fmap2)
export(fmap_element_wise)
export(fwhich)
export(mapreduce)
export(typeof)
exportClasses(FileArray)
exportClasses(FileArrayProxy)
exportMethods(apply)
exportMethods(mapreduce)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setRefClass)
importFrom(methods,signature)
useDynLib(filearray, .registration = TRUE)
