# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

FARR_collapse <- function(filebase, dim, keep, cum_part, array_type, method = 1L, remove_na = FALSE, scale = 1.0) {
    .Call(`_filearray_FARR_collapse`, filebase, dim, keep, cum_part, array_type, method, remove_na, scale)
}

FARR_collapse_complex <- function(filebase, dim, keep, cum_part, method = 1L, remove_na = FALSE, scale = 1.0) {
    .Call(`_filearray_FARR_collapse_complex`, filebase, dim, keep, cum_part, method, remove_na, scale)
}

realToInt64 <- function(x, min_, max_, strict) {
    .Call(`_filearray_realToInt64`, x, min_, max_, strict)
}

cplxToReal2 <- function(x) {
    .Call(`_filearray_cplxToReal2`, x)
}

realToCplx2 <- function(x) {
    .Call(`_filearray_realToCplx2`, x)
}

realToFloat2 <- function(x) {
    .Call(`_filearray_realToFloat2`, x)
}

floatToReal2 <- function(x) {
    .Call(`_filearray_floatToReal2`, x)
}

get_float_na <- function() {
    .Call(`_filearray_get_float_na`)
}

set_buffer_size <- function(size) {
    .Call(`_filearray_set_buffer_size`, size)
}

get_buffer_size <- function() {
    .Call(`_filearray_get_buffer_size`)
}

FARR_meta <- function(filebase) {
    .Call(`_filearray_FARR_meta`, filebase)
}

loc2idx <- function(sliceIdx, dim) {
    .Call(`_filearray_loc2idx`, sliceIdx, dim)
}

locationList <- function(listOrEnv, dim, strict) {
    .Call(`_filearray_locationList`, listOrEnv, dim, strict)
}

schedule <- function(listOrEnv, dim, cum_part_sizes, split_dim, strict) {
    .Call(`_filearray_schedule`, listOrEnv, dim, cum_part_sizes, split_dim, strict)
}

filearray_meta <- function(filebase) {
    .Call(`_filearray_filearray_meta`, filebase)
}

filearray_assign <- function(filebase, value, position_indices) {
    .Call(`_filearray_filearray_assign`, filebase, value, position_indices)
}

filearray_subset <- function(filebase, position_indices, drop = TRUE, use_dimnames = TRUE, reshape = NULL) {
    .Call(`_filearray_filearray_subset`, filebase, position_indices, drop, use_dimnames, reshape)
}

FARR_subset_sequential <- function(filebase, unit_partlen, cum_partsizes, array_type, ret, from = 0L, len = 1L) {
    .Call(`_filearray_FARR_subset_sequential`, filebase, unit_partlen, cum_partsizes, array_type, ret, from, len)
}

FARR_subset2 <- function(filebase, listOrEnv, reshape = NULL, drop = FALSE, use_dimnames = TRUE, thread_buffer = 0L, split_dim = 0L, strict = 1L) {
    .Call(`_filearray_FARR_subset2`, filebase, listOrEnv, reshape, drop, use_dimnames, thread_buffer, split_dim, strict)
}

FARR_buffer_map <- function(input_filebases, output_filebase, map, buffer_nelems, result_nelems = 0L) {
    .Call(`_filearray_FARR_buffer_map`, input_filebases, output_filebase, map, buffer_nelems, result_nelems)
}

FARR_buffer_map2 <- function(input_filebases, map, buffer_nelems) {
    .Call(`_filearray_FARR_buffer_map2`, input_filebases, map, buffer_nelems)
}

FARR_buffer_mapreduce <- function(filebase, map, reduce, buffer_nelems) {
    .Call(`_filearray_FARR_buffer_mapreduce`, filebase, map, reduce, buffer_nelems)
}

FARR_subset_assign_sequential <- function(filebase, unit_partlen, cum_partsizes, array_type, value, from) {
    .Call(`_filearray_FARR_subset_assign_sequential`, filebase, unit_partlen, cum_partsizes, array_type, value, from)
}

FARR_subset_assign2 <- function(filebase, value, listOrEnv, thread_buffer = 2097152L, split_dim = 0L) {
    .Call(`_filearray_FARR_subset_assign2`, filebase, value, listOrEnv, thread_buffer, split_dim)
}

getDefaultNumThreads <- function() {
    .Call(`_filearray_getDefaultNumThreads`)
}

getThreads <- function(max) {
    .Call(`_filearray_getThreads`, max)
}

test_farr_findVarInFrame_ <- function(env, sym_name) {
    .Call(`_filearray_test_farr_findVarInFrame_`, env, sym_name)
}

kinda_sorted <- function(idx, min_, buffer_count) {
    .Call(`_filearray_kinda_sorted`, idx, min_, buffer_count)
}

check_missing_dots <- function(env) {
    .Call(`_filearray_check_missing_dots`, env)
}

reshape_or_drop <- function(x, reshape, drop) {
    .Call(`_filearray_reshape_or_drop`, x, reshape, drop)
}

